import re
from dataclasses import dataclass

from .._types import BaseVideo, BaseVideoMetrics


@dataclass
class VideoMetrics(BaseVideoMetrics):
    """作品指标统计数据"""

    avg_view_proportion: float = None
    """平均观看率"""
    avg_view_second: float = None
    """平均观看时长"""
    bounce_rate_2s: float = None
    """2秒内跳出率"""
    completion_rate: float = None
    completion_rate_5s: float = None
    cover_show: int = None
    danmaku_count: int = None
    """弹幕数量"""
    dislike_count: int = None
    dislike_rate: float = None
    download_count: int = None
    fan_view_proportion: float = None
    """粉丝观看率"""
    homepage_visit_count: int = None
    """主页访问次数"""
    subscribe_count: int = None
    subscribe_rate: float = None
    unsubscribe_count: int = None
    unsubscribe_rate: float = None

    def __post_init__(self):
        super().__post_init__()

        for k, v in self.__dict__.items():
            if v is None:
                continue

            if k.endswith('_proportion'):
                setattr(self, k, round(float(v) * 100, 2))
                continue

            if re.match(r'.*\d+s$', k):
                setattr(self, k, round(float(v) * 100, 2))

        if (
            isinstance(self.avg_view_second, (str, float))
            and self.avg_view_second != ''
        ):
            self.avg_view_second = round(float(self.avg_view_second))

        if isinstance(self.cover_show, (str, int)) and self.cover_show != '':
            self.cover_show = int(self.cover_show)


@dataclass
class Video(BaseVideo):
    """作品对象"""

    metrics: VideoMetrics = None
    """指标统计数据"""
    visibility: dict = None
    """视频可见性"""
    status: int = None
    """1: 正常, 3: 私密"""

    def __post_init__(self):
        if isinstance(self.metrics, dict):
            self.metrics = VideoMetrics(**self.metrics)

        if isinstance(self.visibility, dict):
            self.status = self.visibility.get('status')

        super().__post_init__()

    def as_dict(self):
        """以字典形式返回对象数据"""

        _dict = super().as_dict()
        _dict.pop('visibility')

        return _dict



"""
作品管理页面数据采集
"""

from random import uniform
from time import sleep
from typing import Callable, Literal

from DrissionPage import Chromium
from DrissionPage._pages.mix_tab import MixTab

from .._utils import Utils
from ._dict import Dictionary
from ._types import Video


class Urls:
    works = 'https://creator.douyin.com/creator-micro/content/manage'


class DataPacketUrls:
    works__list = 'https://creator.douyin.com/janus/douyin/creator/pc/work_list'


class Works:
    def __init__(self, browser: Chromium):
        self._browser = browser
        self._timeout = 15
        self._interval_sleep_range = (3, 4.5)

    def __works_enter_check(self, page: MixTab, timeout: float):
        """作品管理页面进入检测"""

        page.listen.start(
            targets=DataPacketUrls.works__list, method='GET', res_type=['Fetch', 'XHR']
        )
        page.get(Urls.works)
        packet = page.listen.wait(timeout=timeout)
        if not packet:
            raise TimeoutError('进入作品管理页面后获取数据超时')

        resp = packet.response.body
        if not isinstance(resp, dict):
            raise TypeError('返回的数据包非预期 dict 类型')
        if 'items' not in resp:
            raise KeyError('数据包中未找到 items 字段')
        items: list[dict] = resp['items']
        if not isinstance(items, list):
            raise TypeError('数据包中 items 字段非预期 list 类型')

        return packet, items, resp.get('max_cursor')

    def __return_video_dict_list(self, video_list: list[Video]):
        """以格式化后的字典列表返回视频对象列表"""

        return [
            Utils.dict_mapping(item.as_dict(), Dictionary.works.video__detail)
            for item in video_list
        ]

    def get__works__list(
        self,
        works_type: Literal['0', '1', '2'],
        timeout: float = None,
        interval_sleep_range: tuple = None,
        show_msg=True,
        as_dict=False,
    ):
        """
        获取作品列表

        Args:
            works_type: 作品类型, 0: 全部, 1: 已发布, 2: 私密
            timeout: 等待数据包超时时间, 默认 15 秒
            interval_sleep_range: 间隔休眠时间范围, 默认 (3, 4.5) 秒
            show_msg: 是否显示提示信息, 默认 True
            as_dict: 是否返回格式化后的字典列表, 默认 False 返回视频对象列表
        Returns:
            作品对象列表
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page = self._browser.new_tab()
        packet, items, first_next_cursor = self.__works_enter_check(page, _timeout)

        if not items:
            page.close()
            return

        video_list: list[Video] = []
        next_cursor = 0
        if packet.request.params.get('status') == works_type:
            video_list = [Video(**item) for item in items]
            next_cursor = first_next_cursor
            if not next_cursor:
                page.close()
                if as_dict is True:
                    return self.__return_video_dict_list(video_list)

                return video_list

        packet_headers = {
            k.lower(): v
            for k, v in packet.request.headers.items()
            if k.lower() in ['referer', 'user-agent', 'x-secsdk-csrf-token']
        }
        packet_query = {
            k: v
            for k, v in packet.request.params.items()
            if k in ['aid', 'count', 'scene', 'device_platform']
        }

        user_agent: str = packet_headers['user-agent']
        browser_name = user_agent[: user_agent.index('/')]
        browser_version = user_agent[user_agent.index('/') + 1 :]

        pub_headers = {**packet_headers}
        pub_query = {
            **packet_query,
            'status': works_type,
            'cookie_enabled': 'true',
            'screen_width': '1920',
            'screen_height': '1080',
            'browser_language': 'en-US',
            'browser_platform': 'Win32',
            'browser_name': browser_name,
            'browser_version': browser_version,
            'browser_online': 'true',
            'timezone_name': 'Asia/Shanghai',
        }

        def query_api(next_cursor: int = 0):
            query_data = {**pub_query, 'max_cursor': next_cursor}
            if not page.get(
                DataPacketUrls.works__list,
                params=query_data,
                headers=pub_headers,
                timeout=_timeout,
            ):
                if show_msg is True:
                    print('- 通过 API 获取作品列表失败')
                return

            try:
                resp: dict = page.response.json()
            except Exception as e:
                if show_msg is True:
                    print(f'- API 返回的数据包解析失败: {e}')
                return

            if 'items' not in resp:
                if show_msg is True:
                    print('- API 返回的数据包中未找到 items 字段')
                return

            items: list[dict] = resp['items']
            if not isinstance(items, list):
                if show_msg is True:
                    print('- API 返回的数据包中 items 字段非预期 list 类型')
                return

            max_cursor: int = resp.get('max_cursor')
            return [Video(**item) for item in items], max_cursor

        curr_page = 1 if not video_list else 2
        _interval_sleep_range = (
            interval_sleep_range
            if isinstance(interval_sleep_range, tuple)
            else self._interval_sleep_range
        )

        page.change_mode('s', go=False)
        for _ in range(500):
            if show_msg is True:
                print(f'- 通过 API 获取第 {curr_page} 页视频列表')

            result = query_api(next_cursor)
            if not result:
                break

            video_list.extend(result[0])
            _next_cursor = result[1]
            if not _next_cursor or _next_cursor == next_cursor or len(video_list) == 0:
                break

            curr_page += 1
            next_cursor = _next_cursor
            sleep(uniform(*_interval_sleep_range))

        page.change_mode('d', go=False)
        page.close()

        if as_dict is True:
            return self.__return_video_dict_list(video_list)

        return video_list

    def get__works__list__normal(
        self,
        works_type: str,
        timeout: float = None,
        interval_sleep_range: tuple = None,
        show_msg=True,
        as_dict=False,
    ):
        """
        获取作品列表 (通过页面访问)

        Args:
            works_type: 作品类型, 例如 全部作品/已发布, 具体可以查看页面
            timeout: 等待数据包超时时间, 默认 15 秒
            interval_sleep_range: 间隔休眠时间范围, 默认 (3, 4.5) 秒
            show_msg: 是否显示提示信息, 默认 True
            as_dict: 是否返回格式化后的字典列表, 默认 False 返回视频对象列表
        Returns:
            作品对象列表
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page = self._browser.new_tab()
        _, items, first_next_cursor = self.__works_enter_check(page, _timeout)

        if not items:
            page.close()
            return

        def get_current_active_tab():
            """获取当前激活的类型选项卡"""
            curr_active_tab = page.ele(
                't:div@@class^tab-item@@class:active-', timeout=3
            )
            if not curr_active_tab:
                raise RuntimeError('未找到当前激活的选项卡')

            return curr_active_tab.text.strip()

        def get_video_list(callback: Callable):
            """获取视频列表数据包"""
            page.listen.start(
                targets=DataPacketUrls.works__list, method='GET', res_type='XHR'
            )
            callback()
            packet = page.listen.wait(timeout=_timeout)
            if not packet:
                raise TimeoutError('获取视频列表数据包超时')

            resp = packet.response.body
            if not isinstance(resp, dict):
                raise TypeError('返回的数据包非预期 dict 类型')
            if 'items' not in resp:
                raise KeyError('数据包中未找到 items 字段')
            items: list[dict] = resp['items']
            if not isinstance(items, list):
                raise TypeError('数据包中 items 字段非预期 list 类型')

            return [Video(**item) for item in items], resp.get('max_cursor')

        video_list: list[Video] = []
        next_cursor = 0
        if get_current_active_tab() != works_type:
            target_tab = page.ele(
                f't:div@@class^tab-item@@text()={works_type}', timeout=3
            )
            if not target_tab:
                raise RuntimeError(f'未找到 [{works_type}] 选项卡')

            video_list, next_cursor = get_video_list(
                lambda: target_tab.click(by_js=True)
            )
        else:
            video_list = [Video(**item) for item in items]
            next_cursor = first_next_cursor

        if not next_cursor:
            page.close()
            if as_dict is True:
                return self.__return_video_dict_list(video_list)

            return video_list

        _interval_sleep_range = (
            interval_sleep_range
            if isinstance(interval_sleep_range, tuple)
            else self._interval_sleep_range
        )

        for _ in range(500):
            if show_msg is True:
                print('- 滚动页面加载视频列表')

            try:
                _video_list, next_cursor = get_video_list(
                    lambda: page.scroll.to_bottom()
                )
                video_list.extend(_video_list)
                if not next_cursor:
                    break
            except Exception as e:
                if show_msg is True:
                    print(f'- 滚动页面加载视频列表出错: {e}')
                break

            sleep(uniform(*_interval_sleep_range))

        page.close()

        if as_dict is True:
            return self.__return_video_dict_list(video_list)

        return video_list



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-02-22
Author: Martian Bugs
Description: 内容管理模块数据采集
"""

from DrissionPage import Chromium

from .works import Works


class Content:
    def __init__(self, browser: Chromium):
        self._browser = browser

        self._works = None

    @property
    def works(self):
        """作品管理页面数据采集"""

        if self._works is None:
            self._works = Works(self._browser)

        return self._works



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-02-23
Author: Martian Bugs
Description: 数据中心模块数据采集
"""

from DrissionPage import Chromium

from .content import Content


class Data:
    def __init__(self, browser: Chromium):
        self._browser = browser

        self._content = None

    @property
    def content(self):
        if self._content is None:
            self._content = Content(self._browser)

        return self._content



class Works:
    video__detail = {
        '视频ID': 'id',
        '视频标题': 'description',
        '封面链接': 'cover',
        '发布时间': 'create_time',
        '发布时间_时间戳': 'create_timestamp',
        '最后更新时间': 'metrics_offline_update_time',
        '最后更新时间_时间戳': 'metrics_offline_update_timestamp',
        '用户ID': 'user_id',
        '视频时长': 'duration',
        '评论数': 'comment_count',
        '评论率': 'comment_rate',
        '收藏数': 'favorite_count',
        '收藏率': 'favorite_rate',
        '点赞量': 'like_count',
        '点赞率': 'like_rate',
        '分享数': 'share_count',
        '分享率': 'share_rate',
        '播放量': 'view_count',
    }


class BaseDictionary:
    works = Works()



from .._dict import BaseDictionary


class Works:
    video__detail = {
        **BaseDictionary.works.video__detail,
        '视频状态': 'status',
        '平均播放占比': 'avg_view_proportion',
        '平均播放时长': 'avg_view_second',
        '2秒跳出率': 'bounce_rate_2s',
        '完播率': 'completion_rate',
        '5秒完播率': 'completion_rate_5s',
        '封面浏览量': 'cover_show',
        '弹幕量': 'danmaku_count',
        '不喜欢量': 'dislike_count',
        '不喜欢率': 'dislike_rate',
        '下载量': 'download_count',
        '粉丝播放占比': 'fan_view_proportion',
        '主页浏览量': 'homepage_visit_count',
        '关注量': 'subscribe_count',
        '关注率': 'subscribe_rate',
        '取关量': 'unsubscribe_count',
        '取关率': 'unsubscribe_rate',
    }


class Dictionary:
    works = Works()



from datetime import datetime


class Utils:
    @staticmethod
    def timestamp_to_str(timestamp: int | str, pattern='%Y-%m-%d %H:%M:%S'):
        """
        将时间戳转为日期时间字符串

        Args:
            timestamp: 时间戳
            pattern: 日期时间格式
        Returns:
            日期时间字符串
        """

        return datetime.fromtimestamp(int(timestamp)).strftime(pattern)

    @staticmethod
    def seconds_to_time(seconds: int):
        """
        秒数转为时间字符串

        Args:
            seconds: 秒数
        Returns:
            时间字符串, 格式为 'xx:xx'
        """

        m, s = divmod(seconds, 60)
        _, m = divmod(m, 60)
        return f'{m:02d}:{s:02d}'

    @staticmethod
    def dict_mapping(data: dict, dict_table: dict[str, str]):
        """
        字典表字段映射

        Args:
            data: 待映射的字典
            dict_table: 字典表
        """

        result = {}
        for text, key in dict_table.items():
            result[text] = data.get(key)

        return result



"""
内容分析页面数据采集
"""

from functools import partial
from random import uniform
from time import sleep
from typing import Callable

from DrissionPage import Chromium

from .._utils import Utils
from ._dict import Dictionary
from ._types import Video


class Urls:
    content_analysis = 'https://creator.douyin.com/creator-micro/data-center/content'


class DataPacketUrls:
    works__overview = (
        'https://creator.douyin.com/janus/douyin/creator/data/item_analysis/overview'
    )
    """投稿数据概览"""
    works__list = 'https://creator.douyin.com/web/api/creator/item/list'


class Content:
    def __init__(self, browser: Chromium):
        self._browser = browser
        self._timeout = 15
        self._interval_sleep_range = (3, 4.5)

    def get__works__list(
        self,
        timeout: float = None,
        interval_sleep_range: tuple = None,
        show_msg=True,
        as_dict=False,
    ):
        """
        获取作品/投稿列表

        Args:
            timeout: 超时时间，默认 15 秒
            interval_sleep_range: 间隔休眠时间范围，默认 (3, 4.5) 秒
            show_msg: 是否显示内部日志信息
            as_dict: 是否返回格式化后的字典列表, 默认 False 返回视频对象列表
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout

        page = self._browser.new_tab()
        page.listen.start(
            targets=DataPacketUrls.works__overview, method='POST', res_type='XHR'
        )
        page.get(Urls.content_analysis)
        if not page.listen.wait(timeout=_timeout):
            raise TimeoutError('进入内容分析页面获取投稿概览数据超时')

        target_btn = page.ele('t:span@@text()=投稿列表', timeout=3)
        if not target_btn:
            raise RuntimeError('未找到 [投稿列表] 按钮')

        def get_video_list(callback: Callable):
            """获取视频列表"""

            page.listen.start(
                targets=DataPacketUrls.works__list, method='GET', res_type='XHR'
            )
            callback()
            packet = page.listen.wait(timeout=_timeout)
            if not packet:
                raise TimeoutError('获取视频列表超时')

            resp: dict = packet.response.body
            if not isinstance(resp, dict):
                raise TypeError('视频列表数据包格式非预期的 dict 类型')

            if 'items' not in resp:
                raise KeyError('数据包中未找到 items 字段')

            items: list[dict] = resp['items']
            if not isinstance(items, list):
                raise TypeError('数据包中的 items 字段非预期的 list 类型')

            return [Video(**item) for item in items], resp.get('has_more')

        get_video_list(lambda: target_btn.click(by_js=True))

        # 修改日期范围为 全部
        date_input = page.ele('t:input@@placeholder=开始日期', timeout=3)
        if not date_input:
            raise RuntimeError('未找到日期输入框')
        date_input.click(by_js=True)
        all_date_btn = page.ele(
            't:div@@class^douyin-creator-pc-datepicker-quick-control@@text()=全部',
            timeout=3,
        )
        if not all_date_btn:
            raise RuntimeError('未找到日期选择器中的 [全部] 按钮')

        sleep(2.5)
        video_list, has_more = get_video_list(lambda: all_date_btn.click(by_js=True))

        _interval_sleep_range = (
            interval_sleep_range
            if isinstance(interval_sleep_range, (list, tuple))
            else self._interval_sleep_range
        )

        sleep(1.5)
        page.scroll.to_bottom()
        while has_more:
            if show_msg is True:
                print('- 滚动表格加载更多视频列表')

            container = page.ele('c:div.douyin-creator-pc-table-body', timeout=3)
            if not container:
                raise RuntimeError('未找到视频列表容器元素')

            try:
                _video_list, has_more = get_video_list(
                    partial(container.scroll.to_bottom)
                )
                video_list.extend(_video_list)
                if not has_more:
                    break
            except Exception as e:
                if show_msg is True:
                    print(f'加载更多视频列表失败: {e}')
                break

            sleep(uniform(*_interval_sleep_range))

        page.close()

        if as_dict is True:
            return [
                Utils.dict_mapping(item.as_dict(), Dictionary.works.video__detail)
                for item in video_list
            ]

        return video_list



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-02-22
Author: Martian Bugs
Description: 数据采集器
"""

from DrissionPage import Chromium, ChromiumOptions

from ._login import Login
from .content.content import Content
from .data.data import Data


class Collector:
    """采集器. 使用之前请先调用 `connect_browser` 方法连接浏览器."""

    def __init__(self):
        self._content = None
        self._data = None

    def connect_browser(self, port: int):
        """
        连接浏览器

        Args:
            port: 浏览器调试端口号
        """

        chrome_options = ChromiumOptions(read_file=False)
        chrome_options.set_local_port(port=port)

        self.browser = Chromium(addr_or_opts=chrome_options)

        self._login_utils = Login(self.browser)

    def login_by_cookie(
        self,
        uid: str,
        cookies: dict,
        local_storage: dict = None,
        session_storage: dict = None,
    ):
        """通过 cookie 登录账号"""

        return self._login_utils.login_by_cookie(
            uid=uid,
            cookies=cookies,
            local_storage=local_storage,
            session_storage=session_storage,
        )

    @property
    def content(self):
        """内容管理模块数据采集"""

        if self._content is None:
            self._content = Content(self.browser)

        return self._content

    @property
    def data(self):
        """数据管理模块数据采集"""

        if self._data is None:
            self._data = Data(self.browser)

        return self._data



from typing import Callable

from DrissionPage import Chromium
from DrissionPage._pages.mix_tab import MixTab


class Urls:
    home = 'https://creator.douyin.com/creator-micro/home'


class DataPacketUrls:
    qrcode = 'https://sso.douyin.com/get_qrcode'
    """扫码登录二维码"""
    account_info = 'https://creator.douyin.com/web/api/media/user/info'


class Login:
    def __init__(self, browser: Chromium):
        self._browser = browser

    def _wait__account_info(
        self, page: MixTab, callback: Callable, timeout: float = None
    ):
        """登录账号信息数据包"""

        _timeout = timeout if isinstance(timeout, (int, float)) else 8

        page.listen.start(
            targets=DataPacketUrls.account_info, method='GET', res_type='Fetch'
        )
        callback()
        packet = page.listen.wait(timeout=_timeout)
        if not packet:
            return

        resp = packet.response.body
        if not isinstance(resp, dict):
            raise RuntimeError('账号信息数据包非预期的 dict 类型')

        if 'user' not in resp:
            raise RuntimeError('账号信息数据包中未找到 user 字段')

        user = resp['user']
        if not isinstance(user, dict):
            raise RuntimeError('账号信息数据包中 user 字段非预期的 dict 类型')

        return user

    def login_by_cookie(
        self,
        uid: str,
        cookies: dict,
        local_storage: dict = None,
        session_storage: dict = None,
        timeout: float = None,
    ):
        """
        通过 cookie 登录

        Args:
            timeout: 判断是否已登录的数据包返回超时时间, 默认为 8 秒
        Returns:
            如果登录成功将返回用户信息
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else 8

        page = self._browser.latest_tab
        page.listen.start(targets=DataPacketUrls.qrcode, method='GET', res_type='XHR')
        page.get(Urls.home)
        page.listen.wait(timeout=_timeout)

        # 不管是否成功获取到二维码数据包，都需要设置 cookies
        page.set.cookies(cookies)

        if isinstance(local_storage, dict):
            for k, v in local_storage.items():
                page.set.local_storage(k, v)

        if isinstance(session_storage, dict):
            for k, v in session_storage.items():
                page.set.session_storage(k, v)

        account_info = self._wait__account_info(
            page, lambda: page.get(Urls.home), timeout=_timeout
        )
        if not account_info:
            raise RuntimeError('登录失败，请检查账号有效性')

        if (_uid := account_info.get('uid')) != uid:
            raise RuntimeError(
                f'登录异常, 登录已登录 uid<{_uid}> 与预期 uid<{uid}> 不一致'
            )

        return account_info



from dataclasses import dataclass, fields

from ._utils import Utils


class FilteredDataclass(type):
    """过滤kwargs中多余的键"""

    def __call__(cls, *args, **kwargs):
        valid_fields = {f.name for f in fields(cls)}
        kwargs = {k: v for k, v in kwargs.items() if k in valid_fields}
        return super().__call__(*args, **kwargs)


class BaseType(metaclass=FilteredDataclass):
    pass


@dataclass
class BaseVideoMetrics(BaseType):
    """作品指标统计数据基础类"""

    comment_count: int = None
    comment_rate: float = None
    favorite_count: int = None
    favorite_rate: float = None
    like_count: int = None
    like_rate: float = None
    share_count: int = None
    share_rate: float = None
    view_count: int = None

    def __post_init__(self):
        for k, v in self.__dict__.items():
            if v == '' or not isinstance(v, (int, str)):
                setattr(self, k, None)
                continue

            if k.endswith('_count'):
                setattr(self, k, int(v))
                continue

            if k.endswith('_rate'):
                setattr(self, k, round(float(v) * 100, 2))
                continue

    def as_dict(self):
        """以字典形式返回对象数据"""

        return self.__dict__.copy()


@dataclass
class BaseVideo(BaseType):
    """作品对象基础类"""

    id: str = None
    """作品ID"""
    cover: str = None
    create_time: str = None
    create_timestamp: int = None
    description: str = None
    """视频标题"""
    metrics: BaseVideoMetrics = None
    """指标统计数据"""
    metrics_offline_update_time: str = None
    metrics_offline_update_timestamp: int = None
    user_id: str = None
    type: int = None
    video_info: dict = None
    duration: str = None

    def __post_init__(self):
        for k in ['id', 'user_id']:
            v = getattr(self, k)
            if not isinstance(v, int):
                continue
            setattr(self, k, str(v))

        if isinstance(self.cover, dict) and 'url_list' in self.cover:
            self.cover = self.cover['url_list'][0]

        for k in ['create_time', 'metrics_offline_update_time']:
            v = getattr(self, k)
            if not isinstance(v, (int, str)):
                continue
            v = int(v)
            setattr(self, k, Utils.timestamp_to_str(v))
            setattr(self, f'{k}stamp', v * 1000)

        if isinstance(self.metrics, dict):
            self.metrics = BaseVideoMetrics(**self.metrics)

        if (
            isinstance(self.video_info, dict)
            and 'duration' in self.video_info
            and isinstance((duration := self.video_info['duration']), (int, str))
        ):
            self.duration = Utils.seconds_to_time(round(int(duration) / 1000))

    def as_dict(self):
        """以字典形式返回对象数据"""

        _dict = self.__dict__.copy()
        _dict.pop('metrics')
        _dict.pop('video_info')

        _dict.update(self.metrics.as_dict())

        return _dict
